/******************************************************************************
 *
 * DAB-Embedded, 2016
 ******************************************************************************/

/******************************************************************************
 *                                                                            *
 * This module connects the an Atmel EBI bus to Avalon Switch Fabric          *
 *  It acts as both a master  for the avalon swtich fabric                    *
 *  and a slave to the master peripheral                                      *
 *                                                                            *
 ******************************************************************************/
module owt_nios_ebi_bus_to_avalon_bridge_0 (

    // Inputs
    clk,
    reset,

    // Inputs from Avalon Switch Fabric
    avalon_readdata,
    avalon_waitrequest,

    // Inputs from the master peripheral
    address,
    chip_select,
    read,
    write,

    // Bidirectionals
    data_bus,

    // Outputs
    // Output to Avalon Switch Fabric
    avalon_address,
    avalon_byteenable,
    avalon_read,
    avalon_write,
    avalon_writedata,

    // Outputs to master peripheral
    acknowledge
);


/*****************************************************************************
 *                           Parameter Declarations                          *
 *****************************************************************************/

parameter    AW    = 11;    // Address width
parameter    DW    = 31;    // Data width
parameter    BW    = 3;     // Byte enable width

/*****************************************************************************
 *                             Port Declarations                             *
 *****************************************************************************/
// Inputs
input                       clk;
input                       reset;

input                       avalon_waitrequest;
input            [DW: 0]    avalon_readdata;

input            [AW: 0]    address;
input                       chip_select;
input                       write;
input                       read;

// Bidirectionals
inout         [15: 0]       data_bus;

// Outputs
output        [AW: 0]       avalon_address;
output        [BW: 0]       avalon_byteenable;
output                      avalon_read;
output                      avalon_write;
output        [DW: 0]       avalon_writedata;

output                      acknowledge;



reg  [BW:0]     i_byte_enable;
reg             i_read_n;
reg             i_write_n;
wire [15 : 0]   data_bus;
reg  [DW : 0]   avalon_writedata;
reg  [15 : 0]   data_bus_reg;

//

always @(posedge clk)
begin
    if ( !chip_select ) begin
        if (address[1] == 1) begin
            i_byte_enable <= 4'b1100;
        end else begin
            i_byte_enable <= 4'b0011;
        end        
    end else begin
         i_byte_enable <= 4'b0000; 
    end
end // proc


always @(posedge clk)
begin
    if ( !read & !chip_select ) begin
         i_read_n <= 1;
    end else begin
         i_read_n <= 0;
    end
end // proc

always @(posedge clk)
begin
    if ( !write & !chip_select ) begin
         i_write_n <= 1;
         avalon_writedata[15 : 0] <= data_bus;
         avalon_writedata[31 : 16] <= data_bus;
    end else begin
         i_write_n <= 0;
    end
end // proc

always @(posedge clk)
begin
    if (!read & !chip_select) begin
        if (!avalon_waitrequest) begin
            if (address[1] == 1) begin
                data_bus_reg <= avalon_readdata[31 : 16];
            end else begin
                data_bus_reg <= avalon_readdata[15 : 0];
            end
        end
    end else begin
        data_bus_reg <= 16'hZZZZ;
    end
end // proc
 
/*****************************************************************************
 *                            Combinational Logic                            *
 *****************************************************************************/
assign avalon_address       = address;
assign avalon_byteenable    = i_byte_enable;
assign avalon_write         = i_write_n;
assign avalon_read          = i_read_n;
assign acknowledge          = ~avalon_waitrequest & (avalon_read | avalon_write);
assign data_bus             = data_bus_reg;

/*****************************************************************************
 *                              Internal Modules                             *
 *****************************************************************************/

endmodule

